import requests

import vdcpy.config as config
import vdcpy.site as vdc_site


def get_files(
    file_id=None, name=None, created_by=None, format=None, visibility=None,
    work=None):
    """
    Get a list of files from the VDC API.

    Parameters
    ----------
    file_id : str
        The ID of the file to retrieve.
    name : str
        The name of the file to retrieve.
    created_by : str
        The email address of the user who created the file.
    format : str
        The format of the file to retrieve.
    visibility : str
        The visibility of the file to retrieve.
    work : str
        The ID of the work to which the file belongs.
    
    Returns
    -------
    file_list : list
        A list of files.
    """
    # Get a list of sites.
    site_list = vdc_site.get_sites()
    file_list_all_sites = []
    for site_info in site_list:
        query = f'http://{site_info["host"]}:5000/{config.FILE_NAMESPACE}'
        if file_id is not None:
            query += f'?id={file_id}'
        if name is not None:
            if file_id is None:
                query += f'?name={name}'
            else:
                query += f'&name={name}'
        if created_by is not None:
            if file_id is None and name is None:
                query += f'?created_by={created_by}'
            else:
                query += f'&created_by={created_by}'
        if format is not None:
            if file_id is None and name is None and created_by is None:
                query += f'?format={format}'
            else:
                query += f'&format={format}'
        if visibility is not None:
            if file_id is None and name is None and created_by is None \
                and format is None:
                query += f'?visibility={visibility}'
            else:
                query += f'&visibility={visibility}'
        if work is not None:
            if file_id is None and name is None and created_by is None \
                and format is None and visibility is None:
                query += f'?work={work}'
            else:
                query += f'&work={work}'

        try:
            response = requests.get(query)
            if response.status_code == 200:
                file_list = response.json()
                for file_info in file_list:
                    file_info['host'] = site_info['name']
                file_list_all_sites += file_list
            else:
                print(
                    f'Error getting files from {site_info["name"]} ' + \
                    f'({site_info["host"]}).' + \
                    f' Status code: {response.status_code}'
                )
        except requests.exceptions.ConnectionError:
            print(
                f'Error connecting to {site_info["name"]} ' + \
                f'({site_info["host"]}).'
            )
    return file_list_all_sites


def add_file(work_id, file_path, auth_token=None):
    """
    Add a file to a work.

    Parameters
    ----------
    work_id : str
        The ID of the work to which the file should be added.
    file_path : str
        The path to the file to be added.
    auth_token : str
        The authentication token to use.
    """
    query = f'{config.API_URL}/{config.FILE_NAMESPACE}/{work_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    # Get filename from file_path
    if '/' in file_path:
        filename = file_path.split('/')[-1]
    else:
        filename = file_path.split('\\')[-1]
    # Post with requests the query and the file
    with open(file_path, 'rb') as file:
        response = requests.post(
            query, files={'file': (filename, file)}, headers=headers
        )
    if response.status_code != 201:

        raise Exception(f'Error adding file: {response.status_code}')
    return response.json()


def delete_file(file_id, auth_token=None):
    """
    Delete a file from the VDC API.

    Parameters
    ----------
    file_id : str
        The ID of the file to delete.
    auth_token : str
        The authentication token to use.
    """
    query = f'{config.API_URL}/{config.FILE_NAMESPACE}/{file_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.delete(query, headers=headers)
    if response.status_code != 204:
        raise Exception(f'Error deleting file: {response.status_code}')


def download_file(file_id, file_path):
    """
    Download a file from the VDC API.

    Parameters
    ----------
    file_id : str
        The ID of the file to download.
    file_path : str
        The path to which the file should be downloaded.
    
    Returns
    -------
    file_path : str
        The path to which the file was downloaded.
    """
    query = f'{config.API_URL}/{config.FILE_NAMESPACE}?id={file_id}'
    response = requests.get(query)
    if response.status_code == 200 and response.json():
        file_url = response.json()[0]['download_url']
        if file_url is not None:
            # Download the file into the file_path
            response = requests.get(file_url)
            if response.status_code == 200:
                with open(file_path, 'wb') as file:
                    file.write(response.content)
                return file_path
            else:
                raise Exception(
                    f'Error downloading file: {response.status_code}')
    else:
        raise Exception(
            f'Error getting file information: {response.status_code}')
