import requests

import vdcpy.config as config
import vdcpy.site as vdc_site

def get_users(
    user_id=None, name=None, email=None, organization=None, position=None):
    """
    Get a list of users from the VDC API.

    Parameters
    ----------
    user_id : str
        The user ID of the user to get.
    name : str
        The name of the user to get.
    email : str
        The email of the user to get.
    organization : str
        The organization of the user to get.
    position : str
        The position of the user to get.

    Returns
    -------
    user_list : list
        A list of users.
    """
    # Get a list of sites.
    site_list = vdc_site.get_sites()
    # Get a list of users.
    user_list_all_host = []
    for site_info in site_list:
        query = f'http://{site_info["host"]}:5000/{config.USER_NAMESPACE}'
        if user_id is not None:
            query += f'?id={user_id}'
        if name is not None:
            if user_id is None:
                query += f'?name={name}'
            else:
                query += f'&name={name}'
        if email is not None:
            if user_id is None and name is None:
                query += f'?email={email}'
            else:
                query += f'&email={email}'
        if organization is not None:
            if user_id is None and name is None and email is None:
                query += f'?organization={organization}'
            else:
                query += f'&organization={organization}'
        if position is not None:
            if user_id is None and name is None and email is None and \
                organization is None:
                query += f'?position={position}'
            else:
                query += f'&position={position}'

        try:
            response = requests.get(query, timeout=5)
            if response.status_code == 200:
                user_list = response.json()
                for user_info in user_list:
                    user_info['host'] = site_info['name']
                user_list_all_host += user_list
            else:
                print(
                    f'Error getting users from {site_info["name"]} ' + \
                    f'({site_info["host"]}).' + \
                    f' Status code: {response.status_code}'
                )
        except requests.exceptions.ConnectionError:
            print(
                f'Error connecting to {site_info["name"]} ' + \
                f'({site_info["host"]}).'
            )
        except requests.exceptions.ReadTimeout:
            print(
                f'Read timed out from {site_info["name"]} ' + \
                f'({site_info["host"]}).'
            )
    return user_list_all_host


def add_user(user_info, auth_token=None):
    """
    Add a user to the VDC API.

    Parameters
    ----------
    user_info : dict
        The user information to add.
    """

    query = f'{config.API_URL}/{config.USER_NAMESPACE}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.post(
        query, json=user_info, headers=headers, timeout=5
    )
    if response.status_code == 201:
        return response.json()
    else:
        raise Exception(
            f'Error adding user: {response.status_code} - {response.json()}'
        )


def delete_user(user_id, auth_token=None):
    """
    Delete a user from the VDC API.

    Parameters
    ----------
    user_id : str
        The user ID of the user to delete.
    """
    query = f'{config.API_URL}/{config.USER_NAMESPACE}/{user_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.delete(query, headers=headers, timeout=5)
    if response.status_code == 204:
        return True
    else:
        raise Exception(
            f'Error deleting user: {response.status_code} - {response.json()}'
        )


def update_user(user_id, user_info, auth_token=None):
    """
    Update a user in the VDC API.

    Parameters
    ----------
    user_id : str
        The user ID of the user to update.
    user_info : dict
        The user information to update.
    """
    query = f'{config.API_URL}/{config.USER_NAMESPACE}/{user_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.put(query, json=user_info, headers=headers, timeout=5)
    if response.status_code == 200:
        return response.json()
    else:
        raise Exception(
            f'Error updating user: {response.status_code} - {response.json()}'
        )
