from wsgiref import headers
import requests

import vdcpy.config as config
import vdcpy.site as vdc_site


def get_works(
    work_id=None, title=None, abstract=None, genre=None, visibility=None,
    project=None, created_by=None):
    """
    Get a list of works from the VDC API.

    Parameters
    ----------
    work_id : str
        The ID of the work to retrieve.
    title : str
        The title of the work to retrieve.
    abstract : str
        The abstract of the work to retrieve.
    genre : str
        The genre of the work to retrieve.
    visibility : str
        The visibility of the work to retrieve.
    project : str
        The ID of the project to which the work belongs.
    created_by : str
        The email address of the user who created the work.

    Returns
    -------
    work_list_all_sites : list
        A list of works.

    """
    # Get a list of sites.
    site_list = vdc_site.get_sites()
    work_list_all_sites = []
    for site_info in site_list:
        query = f'http://{site_info["host"]}:5000/{config.WORK_NAMESPACE}'
        if work_id is not None:
            query += f'?id={work_id}'
        if title is not None:
            if work_id is None:
                query += f'?title={title}'
            else:
                query += f'&title={title}'
        if abstract is not None:
            if work_id is None and title is None:
                query += f'?abstract={abstract}'
            else:
                query += f'&abstract={abstract}'
        if genre is not None:
            if work_id is None and title is None and abstract is None:
                query += f'?genre={genre}'
            else:
                query += f'&genre={genre}'
        if visibility is not None:
            if work_id is None and title is None and abstract is None and genre is None:
                query += f'?visibility={visibility}'
            else:
                query += f'&visibility={visibility}'
        if project is not None:
            if work_id is None and title is None and abstract is None and genre is None and visibility is None:
                query += f'?project={project}'
            else:
                query += f'&project={project}'
        if created_by is not None:
            if work_id is None and title is None and abstract is None and genre is None and visibility is None and project is None:
                query += f'?created_by={created_by}'
            else:
                query += f'&created_by={created_by}'
        try:
            response = requests.get(query)
            if response.status_code == 200:
                work_list = response.json()
                for work_info in work_list:
                    work_info['host'] = site_info['name']
                work_list_all_sites += work_list
            else:
                print(
                        f'Error getting works from {site_info["name"]} ' + \
                        f'({site_info["host"]}).' + \
                        f' Status code: {response.status_code}'
                    )
        except requests.exceptions.ConnectionError:
            print(
                f'Error connecting to {site_info["name"]} ' + \
                f'({site_info["host"]}).'
            )
    return work_list_all_sites


def add_work(work_info, auth_token=None):
    """
    Add a work to the VDC API.
    """
    query = f'{config.API_URL}/{config.WORK_NAMESPACE}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.post(query, json=work_info, headers=headers)
    if response.status_code == 201:
        return response.json()
    else:
        raise Exception(
            f'Error adding work: {response.status_code} - {response.json()}'
        )


def update_work(work_id, work_info, auth_token=None):
    """
    Update a work in the VDC API.
    """
    query = f'{config.API_URL}/{config.WORK_NAMESPACE}/{work_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.put(query, json=work_info, headers=headers)

    if response.status_code == 200:
        return response.json()
    else:
        raise Exception(
            f'Error updating work: {response.status_code} - {response.json()}'
        )


def delete_work(work_id, auth_token=None):
    """
    Delete a work from the VDC API.
    """
    query = f'{config.API_URL}/{config.WORK_NAMESPACE}/{work_id}'
    if auth_token is not None:
        headers = {'Authorization': auth_token}
    else:
        headers = None
    response = requests.delete(query, headers=headers)
    if response.status_code == 204:
        return True
    else:
        raise Exception(f'Error deleting work: {response.status_code}')
